/*
 * Copyright (C) 2009 Red Hat, Inc.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <prtypes.h>

#ifndef cmcertext_h
#define cmcertext_h

struct NSSInitContextStr;

struct cm_store_entry;
void cm_certext_build_csr_extensions(struct cm_store_entry *entry,
				     struct NSSInitContextStr *ctx,
				     unsigned char **encoded, size_t *length);

/* Validate a V2 template spec */
PRBool cm_ms_template_valid(char *template_spec);

#endif
